#include <alsa/sound/type_compat.h>
#include <alsa/sound/uapi/hdspm.h>
#ifndef __hdspm_type_defined
#define __hdspm_type_defined
typedef struct hdspm_peak_rms hdspm_peak_rms_t;
typedef struct hdspm_config_info hdspm_config_info_t;
typedef struct hdspm_version hdspm_version_t;
typedef struct hdspm_channelfader snd_hdspm_channelfader_t;
typedef struct hdspm_mixer hdspm_mixer_t;
#endif
